/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2002 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.jorm;

import org.apache.tools.ant.types.DTDLocation;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.lib.Log;
import org.objectweb.medor.lib.TestMedorHelper;
import org.objectweb.medor.optim.api.QueryRewriter;
import org.objectweb.medor.query.api.QueryTree;
import org.objectweb.medor.query.jorm.lib.ClassExtent;
import org.objectweb.medor.query.lib.QueryTreePrinter;
import org.objectweb.jorm.util.io.lib.DirJavaExplorer;
import org.objectweb.jorm.compiler.lib.JormCompiler;
import org.objectweb.jorm.compiler.api.JormCompilerParameter;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Collections;

/**
 * @author S.Chassande-Barrioz
 */
public abstract class TestJormHelper extends TestMedorHelper {
	private static String mapperName;
	private static String projectName;
	private static String miMappingFactory;
	private static String mappingParser;
	private static String pdPath;
	private static String dtdPath;

	public static final String DEFAULT_PNAME_FIELD_NAME = "object";
	public static final String PROJECT_NAME = "test";

	static {
		Logger localLogger = getLoggerFactory()
			.getLogger(Log.MEDOR_PREFIX + ".jorm");
		mapperName = System.getProperty(
			"medor.jorm.mapperName", "rdb");
		localLogger.log(BasicLevel.DEBUG, "MapperName = " + mapperName);

		projectName = System.getProperty(
			"medor.jorm.projectName", "");
		localLogger.log(BasicLevel.DEBUG, "ProjectName = " + projectName);

		miMappingFactory = System.getProperty(
			"medor.jorm.miMappingName",
			"org.objectweb.jorm.metainfo.rdb.BasicRDBMappingFactory");
		localLogger.log(
			BasicLevel.DEBUG, "MIMappingFactory = " + miMappingFactory);

		mappingParser = System.getProperty(
			"medor.jorm.miMappingName",
			"org.objectweb.jorm.xml2mi.rdb.RDBParser");
		localLogger.log(BasicLevel.DEBUG, "MappingParser = " + mappingParser);

		pdPath = System.getProperty(
			"medor.jorm.pd.path", "./");
		localLogger.log(BasicLevel.DEBUG, "pd Path = " + pdPath);
		dtdPath = System.getProperty(
			"medor.jorm.dtd.path", "./");
		localLogger.log(BasicLevel.DEBUG, "dtd Path = " + dtdPath);
	}

	public Manager manager = null;
	public PMapper mapper = null;
	public JormCompiler compiler = null;

	public class Example {
		public QueryTree qt;
		public Class clazz;
		public ClassExtent extent;
		public String pnameFieldName;
	}

	public TestJormHelper(String testName, String loggerName) {
		super(testName, loggerName);
		try {
			compiler = new JormCompiler();
			compiler.getCompilerConfigurator().configure();
            mapper = newMappper();
            mapper.setMapperName(getMapperName());
			LoggerFactory loggerFactory = getLoggerFactory();
			compiler.getCompilerConfigurator().setLoggerFactory(loggerFactory);
			compiler.setupLogger();
			compiler.setupMIManager();
			manager = compiler.getMIManager();
			//compiler.setupVerifier(); currently the verifier is not availlable
			compiler.setupGenerator();
			JormCompilerParameter cp = compiler.getCompilerParameter();
			DirJavaExplorer exp = new DirJavaExplorer();
			exp.addPath(getPDPath());
			cp.setClasspath(exp);
			cp.setProjectName(getProjectName());
			DTDLocation dtdl = new DTDLocation();
			dtdl.setPublicId("-//ObjectWeb Consortium//DTD JORM DEFINITIONS 2.0//EN");
			dtdl.setLocation(getDTDPath());
			cp.getDtdLocations().add(dtdl);
			compiler.setupParser();
		}
		catch (PException e) {
			fail(e.getMessage());
		}
		compiler.getCompilerParameter().setProjectName(PROJECT_NAME);
	}

	public String getPDPath() {
		return pdPath;
	}

	public String getDTDPath() {
		return dtdPath;
	}

	public String getMapperName() {
		return mapperName;
	}

	public String getProjectName() {
		return projectName;
	}

	public String getMIMappingFactory() {
		return miMappingFactory;
	}

	public String getMappingParser() {
		return mappingParser;
	}

	public abstract PMapper newMappper();

	public abstract QueryRewriter createQueryRewriter();

	protected void setUp() {
	}

	protected void tearDown() {
	}

	public Example getExtent(Collection fileNames, String className) {
		return getExtent(fileNames, className, DEFAULT_PNAME_FIELD_NAME, null);
	}

	public Example getExtent(Collection fileNames,
							 String fqClassName,
							 String pnameFieldName,
							 String[] fieldNames) {

		logger.log(BasicLevel.DEBUG, "getExtentOf the class " + fqClassName);

		logger.log(BasicLevel.DEBUG, "manager=" + manager);
		Collection mos = null;

		Class c = manager.getClass(fqClassName);

		if (c == null) {
			try {
				mos = loadJormMetaInfo(fileNames);
				logger.log(BasicLevel.DEBUG, "Jorm meta information loaded");
			}
			catch (PException e) {
				e.printStackTrace(System.out);
				fail(e.getMessage());
			}
		}
		else {
			mos = Collections.singletonList(c);
		}

		for (Iterator it = mos.iterator(); it.hasNext();) {
			MetaObject mo = (MetaObject) it.next();
			if (mo instanceof Class
				&& fqClassName.equals(((Class) mo).getFQName())) {
				Example e = new Example();
				e.pnameFieldName = pnameFieldName;
				e.clazz = (Class) mo;
				if (manager == null) {
					manager = (Manager) mo.getParent().getParent();
                    throw new RuntimeException("The test must be changed in according to JORM 2.2");
					//mapper.setMetaInfoManager(manager);
				}
				logger.log(BasicLevel.DEBUG, "schema=" + mo.getParent());
				logger.log(BasicLevel.DEBUG, "parent(schema)=" + mo.getParent().getParent());
				try {
					if (fieldNames == null)
						e.extent = new ClassExtent(
							e.clazz, e.clazz.getFQName(), pnameFieldName, false);
					else
						e.extent = new ClassExtent(e.clazz, e.clazz.getFQName(),
							fieldNames, true, pnameFieldName);
					logger.log(BasicLevel.DEBUG, "Extent created for the class "
						+ e.clazz.getName());
					e.extent.setPMapper(mapper, PROJECT_NAME);

					QueryTreePrinter.printQueryTree(e.extent, logger);
					return e;
				}
				catch (MedorException me) {
					me.printStackTrace();
					fail(me.getMessage());
				}
			}
		}
		return null;
	}

	public Example getRewritten(Example e) {
		if (e == null)
			return null;
		try {
			//assign mapper and project names to the JormExtent
			e.extent.setProjectName("test");
			e.extent.setPMapper(this.mapper, PROJECT_NAME);

			QueryRewriter qr = createQueryRewriter();
			logger.log(BasicLevel.DEBUG, "QueryRewriter created");

			e.qt = qr.transform(e.extent);
			logger.log(BasicLevel.DEBUG, "Extent of the class "
				+ e.clazz.getName() + " rewritten");

			QueryTreePrinter.printQueryTree(e.qt, logger);
		}
		catch (MedorException me) {
			me.printStackTrace();
			fail(me.getMessage());
		}
		return e;
	}

	public Collection loadJormMetaInfo(String fileName) throws PException {
		ArrayList al = new ArrayList(1);
		al.add(fileName);
		return loadJormMetaInfo(al);
	}

	public Collection loadJormMetaInfo(Collection fileNames) throws PException {
		Collection mos = compiler.parseFiles(fileNames);
		logger.log(BasicLevel.DEBUG, "Files parsed (" + mos.size() + " item(s) found)");

		mos.iterator();
		/*
		MetaInfoPrinter moprint = new MetaInfoPrinter();
		while (it.hasNext()) {
			MetaObject mo = (MetaObject) it.next();
			moprint.print("", mo, new PrintStreamImpl(logger));
		}
		*/
		return mos;
	}

	public String getFieldName(String alias, String fn) {
		return (alias == null || alias.length() == 0 ? "" : alias + ".") + fn;
	}
}
