/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2002 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, S. Chassande-Barrioz, A. Lefebvre
 */

package org.objectweb.medor.query.rdb.lib;

import junit.framework.Test;
import junit.framework.TestSuite;
import junit.textui.TestRunner;
import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.jorm.mapper.rdb.adapter.PostgresAdapter;
import org.objectweb.jorm.mapper.rdb.adapter.OracleAdapter;
import org.objectweb.medor.filter.api.FieldOperand;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.Operand;
import org.objectweb.medor.expression.lib.BasicOperand;
import org.objectweb.medor.expression.lib.And;
import org.objectweb.medor.filter.lib.BasicFieldOperand;
import org.objectweb.medor.expression.lib.BasicParameterOperand;
import org.objectweb.medor.expression.lib.Concat;
import org.objectweb.medor.expression.lib.Equal;
import org.objectweb.medor.expression.lib.Greater;
import org.objectweb.medor.filter.lib.InCollection;
import org.objectweb.medor.expression.lib.Lower;
import org.objectweb.medor.filter.lib.MemberOf;
import org.objectweb.medor.expression.lib.Mod;
import org.objectweb.medor.expression.lib.Substring;
import org.objectweb.medor.filter.jorm.lib.IsNullPName;
import org.objectweb.medor.lib.Log;
import org.objectweb.medor.lib.TestMedorHelper;
import org.objectweb.medor.query.api.OrderField;
import org.objectweb.medor.query.lib.BasicOrderField;
import org.objectweb.medor.query.rdb.api.QualifiedTable;
import org.objectweb.medor.query.rdb.api.RdbExpField;
import org.objectweb.medor.query.rdb.api.RdbExpQueryLeaf;
import org.objectweb.medor.type.lib.PTypeSpaceMedor;
import org.objectweb.util.monolog.api.BasicLevel;

import java.util.ArrayList;
import java.util.Collection;

public class TestRewrite2SQL extends TestMedorHelper {

	/**
	 * main method to launch the tests manually
	 */
	public static void main(String[] args) {
		TestRunner.run(suite());
	}

	/**
	 * This method creates a TestSuite object with the current tests
	 */
	public static Test suite() {
		return new TestSuite(TestRewrite2SQL.class);
	}

	public TestRewrite2SQL() {
		super("TestRewrite2SQL", Log.MEDOR_PREFIX + ".query.rdb.lib.rewrite2sql");
	}

	public TestRewrite2SQL(String testName) {
		super(testName, Log.MEDOR_PREFIX + ".query.rdb.lib.rewrite2sql");
	}

	protected void setUp() {
	}

	protected void tearDown() {
	}

	public void testAliasesSimple() {
		try {
			//create qualified tables
			QualifiedTable t1 = new BasicQualifiedTable("t1", null);
			QualifiedTable t2 = new BasicQualifiedTable("t2", "al2");
			QualifiedTable[] tables = {t1, t2};
			//create a new leaf
			RdbExpQueryLeaf ql = new BasicRdbExpQueryLeaf(null, tables, "");
			//add fields
			RdbExpField f1 = ql.addRdbField("f1", null, "col1", t1);
			RdbExpField f2 = ql.addRdbField("f2", null, "col2", t2);
			//create a filter to be added to the query leaf
			FieldOperand fieldOp1 = new BasicFieldOperand(f1);
			FieldOperand fieldOp2 = new BasicFieldOperand(f2);
			BasicOperand valueL = new BasicOperand(1013);
			BasicOperand valueR = new BasicOperand("coucou");
			Expression left = new Lower(fieldOp1, valueL);
			Expression right = new Greater(fieldOp2, valueR);
			Expression filter = new And(left, right);
			//assign the filter to the query leaf
			ql.setQueryFilter(filter);
			logger.log(BasicLevel.DEBUG, "testAliasesSimple");
			//compute the SQL query
			logger.log(BasicLevel.DEBUG, ql.getSqlRequest(null));
			assertEquals("Bad sql request",
				"SELECT t1.col1, al2.col2 FROM t1, t2 as al2 WHERE ((t1.col1 < 1013) AND (al2.col2 > 'coucou'))",
				ql.getSqlRequest(null));
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}

	public void testDistinct() {
		try {
			//create qualified tables
			QualifiedTable t1 = new BasicQualifiedTable("t1", null);
			QualifiedTable t2 = new BasicQualifiedTable("t2", "al2");
			QualifiedTable[] tables = {t1, t2};
			//create a new leaf
			RdbExpQueryLeaf ql = new BasicRdbExpQueryLeaf(null, tables, "");
			ql.setDistinct(true);
			//add fields
			RdbExpField f1 = ql.addRdbField("f1", null, "col1", t1);
			RdbExpField f2 = ql.addRdbField("f2", null, "col2", t2);
			//create a filter to be added to the query leaf
			FieldOperand fieldOp1 = new BasicFieldOperand(f1);
			FieldOperand fieldOp2 = new BasicFieldOperand(f2);
			BasicOperand valueL = new BasicOperand(1013);
			BasicOperand valueR = new BasicOperand("coucou");
			Expression left = new Lower(fieldOp1, valueL);
			Expression right = new Greater(fieldOp2, valueR);
			Expression filter = new And(left, right);
			//assigne the filter to the query leaf
			ql.setQueryFilter(filter);
			logger.log(BasicLevel.DEBUG, "testDistinct");
			//computes the SQL query
			logger.log(BasicLevel.DEBUG, ql.getSqlRequest(null));
			assertEquals("Bad sql request",
				"SELECT DISTINCT t1.col1, al2.col2 FROM t1, t2 as al2 WHERE ((t1.col1 < 1013) AND (al2.col2 > 'coucou'))",
				ql.getSqlRequest(null));
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	public void testOrderBy() {
		try {
			//create qualified tables
			QualifiedTable t1 = new BasicQualifiedTable("t1", null);
			QualifiedTable t2 = new BasicQualifiedTable("t2", "al2");
			QualifiedTable[] tables = {t1, t2};
			//create a new leaf
			RdbExpQueryLeaf ql = new BasicRdbExpQueryLeaf(null, tables, "");
			ql.setDistinct(true);
			//add fields
			RdbExpField f1 = ql.addRdbField("f1", null, "col1", t1);
			RdbExpField f2 = ql.addRdbField("f2", null, "col2", t2);
			//create a filter to be added to the query leaf
			FieldOperand fieldOp1 = new BasicFieldOperand(f1);
			FieldOperand fieldOp2 = new BasicFieldOperand(f2);
			BasicOperand valueL = new BasicOperand(1013);
			BasicOperand valueR = new BasicOperand("coucou");
			Expression left = new Lower(fieldOp1, valueL);
			Expression right = new Greater(fieldOp2, valueR);
			Expression filter = new And(left, right);
			//assign the filter to the query leaf
			ql.setQueryFilter(filter);
			//add an OrderBy Field
			OrderField of1 = new BasicOrderField(f1, true);
			OrderField of2 = new BasicOrderField(f2);
			ql.setOrderBy(new OrderField[] {of1, of2});
			logger.log(BasicLevel.DEBUG, "testOrderBy");
			//compute the SQL query
			logger.log(BasicLevel.DEBUG, ql.getSqlRequest(null));
			assertEquals("Bad sql request",
				"SELECT DISTINCT t1.col1, al2.col2 FROM t1, t2 as al2 WHERE ((t1.col1 < 1013) AND (al2.col2 > 'coucou')) ORDER BY t1.col1 DESC, al2.col2 ASC",
				ql.getSqlRequest(null));
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}
	public void testMemberOfParam() {
		try {
			//create qualified tables
			QualifiedTable t1 = new BasicQualifiedTable("t1", null);
			QualifiedTable t2 = new BasicQualifiedTable("t2", "al2");
			QualifiedTable[] table1 = {t1};
			QualifiedTable[] table2 = {t2};
			//create a new leaf
			RdbExpQueryLeaf ql = new BasicRdbExpQueryLeaf(null, table1, "");
			RdbExpQueryLeaf q2 = new BasicRdbExpQueryLeaf(null, table2, "");
			//add fields
			RdbExpField f1 = ql.addRdbField("f1", null, "col1", t1);
			RdbExpField f2 = q2.addRdbField("f2", null, "col2", t2);
			//create a filter to be added to the query leaf
			FieldOperand fieldOp2 = new BasicFieldOperand(f2);
			BasicOperand value = new BasicOperand(1013);
			ArrayList al1 = new ArrayList();
			ArrayList al2 = new ArrayList();
			al1.add(value);
			al2.add(fieldOp2);
			Expression filter = new MemberOf(al1, al2);
			//assign the filter to the query leaf
			ql.setQueryFilter(filter);
			logger.log(BasicLevel.DEBUG, "testMemberOfParam");
			//compute the SQL query
			logger.log(BasicLevel.DEBUG, ql.getSqlRequest(null));
			assertEquals("Bad sql request",
				"SELECT t1.col1 FROM t1 WHERE (1013 IN (SELECT al2.col2 FROM t2 as al2))",
				ql.getSqlRequest(null));
		} catch (Exception e) {
			e.printStackTrace(System.out);
			fail(e.getMessage());
		}
	}

	public void testAliasesWithParam() {
		try {
			//create qualified tables
			QualifiedTable t1 = new BasicQualifiedTable("t1", null);
			QualifiedTable t2 = new BasicQualifiedTable("t2", "al2");
			QualifiedTable[] tables = {t1, t2};
			//create a new leaf
			RdbExpQueryLeaf ql = new BasicRdbExpQueryLeaf(null, tables, "");
			//add fields
			RdbExpField f1 = ql.addRdbField("f1", null, "col1", t1);
			RdbExpField f2 = ql.addRdbField("f2", null, "col2", t2);
			//create a filter to be added to the query leaf
			FieldOperand fieldOp1 = new BasicFieldOperand(f1);
			FieldOperand fieldOp2 = new BasicFieldOperand(f2);
			BasicOperand valueL = new BasicOperand(1013);
			BasicOperand valueR = new BasicOperand("coucou");
			Expression left = new Lower(fieldOp1, valueL);
			Expression right = new Greater(fieldOp2, valueR);
			Expression filter = new And(left, right);
			//assign the filter to the query leaf
			ql.setQueryFilter(filter);
			ParameterOperand fieldOp3 =
				new BasicParameterOperand(PTypeSpaceMedor.STRING, "x");
			Expression left2 = new Lower(fieldOp3, valueL);
			filter = new And(left2, right);
			ql.setQueryFilter(filter);
			ql.setGroupBy(f2);
			logger.log(BasicLevel.DEBUG, "testAliasesWithParam");
			//compute the SQL query
			logger.log(BasicLevel.DEBUG, ql.getSqlRequest(null));
			assertEquals("Bad sql request",
				"SELECT t1.col1, al2.col2 FROM t1, t2 as al2 WHERE ((? < 1013) AND (al2.col2 > 'coucou')) GROUP BY al2.col2",
				ql.getSqlRequest(null));
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	public void testMod() {
		try {
			//create qualified tables
			QualifiedTable t1 = new BasicQualifiedTable("t1", null);
			QualifiedTable t2 = new BasicQualifiedTable("t2", "al2");
			QualifiedTable[] tables = {t1, t2};
			//create a new leaf
			RdbExpQueryLeaf ql = new BasicRdbExpQueryLeaf(null, tables, "");
			//add fields
			RdbExpField f1 = ql.addRdbField("f1", null, "col1", t1);
			RdbExpField f2 = ql.addRdbField("f2", null, "col2", t2);
			//create a filter to be added to the query leaf
			FieldOperand fieldOp1 = new BasicFieldOperand(f1);
			FieldOperand fieldOp2 = new BasicFieldOperand(f2);
			BasicOperand valueL = new BasicOperand(1013);
			Expression mod = new Mod(fieldOp1, valueL);
			Expression right = new Lower(fieldOp2, mod);
			ParameterOperand paramOp3 =
				new BasicParameterOperand(PTypeSpaceMedor.INT, "x");
			Expression mod2 = new Mod(paramOp3, fieldOp1);
			Expression left = new Greater(fieldOp2, mod2);

			Expression filter = new And(left, right);
			//assign the filter to the query leaf
			ql.setQueryFilter(filter);
			logger.log(BasicLevel.DEBUG, "testAliasesWithParam");
			//compute the SQL query
			logger.log(BasicLevel.DEBUG, ql.getSqlRequest(null));
			assertEquals("Bad sql request",
				"SELECT t1.col1, al2.col2 FROM t1, t2 as al2 WHERE ((al2.col2 > mod(?, t1.col1)) AND (al2.col2 < mod(t1.col1, 1013)))",
				ql.getSqlRequest(null));
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

		public void testSubstring() {
		try {
			//create qualified tables
			QualifiedTable t1 = new BasicQualifiedTable("t1", null);
			QualifiedTable[] tables = {t1};
			//create a new leaf
			RdbExpQueryLeaf ql = new BasicRdbExpQueryLeaf(null, tables, "");
			//add fields
			RdbExpField f1 = ql.addRdbField("f1", null, "col1", t1);
			//create a filter to be added to the query leaf
			FieldOperand fieldOp1 = new BasicFieldOperand(f1);
			BasicOperand valueFrom = new BasicOperand(12);
			BasicOperand valueTo = new BasicOperand(124);
			BasicOperand valueComp = new BasicOperand("toto");
			Expression exp = new Substring(fieldOp1, valueFrom, valueTo);
			Expression filter = new Equal(exp, valueComp);
			//assign the filter to the query leaf
			ql.setQueryFilter(filter);
			logger.log(BasicLevel.DEBUG, "testSubstring");
			logger.log(BasicLevel.DEBUG, ql.getSqlRequest(null));
			assertEquals("Bad sql request",
				"SELECT t1.col1 FROM t1 WHERE (substring(t1.col1, 12, 124) = 'toto')",
				ql.getSqlRequest(null));
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	public void testConcat() {
		try {
			//create qualified tables
			QualifiedTable t1 = new BasicQualifiedTable("t1", null);
			QualifiedTable[] tables = {t1};
			//create a new leaf
			RdbExpQueryLeaf ql = new BasicRdbExpQueryLeaf(null, tables, "");
			//add fields
			RdbExpField f1 = ql.addRdbField("f1", null, "col1", t1);
			//create a filter to be added to the query leaf
			FieldOperand fieldOp1 = new BasicFieldOperand(f1);
			BasicOperand valueTo = new BasicOperand("to");
			BasicOperand valueComp = new BasicOperand("toto");
			Expression exp = new Concat(fieldOp1, valueTo);
			Expression filter = new Equal(exp, valueComp);
			//assign the filter to the query leaf
			ql.setQueryFilter(filter);
			logger.log(BasicLevel.DEBUG, "testSubstring");
			//compute the SQL query
			logger.log(BasicLevel.DEBUG, ql.getSqlRequest(null));
			assertEquals("Bad sql request",
				"SELECT t1.col1 FROM t1 WHERE (concat(t1.col1, 'to') = 'toto')",
				ql.getSqlRequest(null));
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	public void testConcatConcat() {
		try {
			//create qualified tables
			QualifiedTable t1 = new BasicQualifiedTable("t1", null);
			QualifiedTable[] tables = {t1};
			//create a new leaf
			RdbExpQueryLeaf ql = new BasicRdbExpQueryLeaf(null, tables, "");
			//add fields
			RdbExpField f1 = ql.addRdbField("f1", null, "col1", t1);
			RdbExpField f2 = ql.addRdbField("f2", null, "col2", t1);
			//create a filter to be added to the query leaf
			FieldOperand fieldOp1 = new BasicFieldOperand(f1);
			FieldOperand fieldOp2 = new BasicFieldOperand(f2);
			BasicOperand valueTo = new BasicOperand("to");
			BasicOperand valueComp = new BasicOperand("toto");
			Expression exp1 = new Concat(fieldOp1, valueTo);
			Expression exp = new Concat(exp1, fieldOp2);
			Expression filter = new Equal(exp, valueComp);
			//assign the filter to the query leaf
			ql.setQueryFilter(filter);
			logger.log(BasicLevel.DEBUG, "testSubstring");
			logger.log(BasicLevel.DEBUG, ql.getSqlRequest(null));
			assertEquals("Bad sql request",
				"SELECT t1.col1, t1.col2 FROM t1 WHERE (concat(concat(t1.col1, 'to'), t1.col2) = 'toto')",
				ql.getSqlRequest(null));
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	public void testEmptyCollection() {
		testEmptyCollection(false);
	}
	public void testParamEmptyCollection() {
		testEmptyCollection(true);
	}
	public void testEmptyCollection(boolean isParameter) {
		try {
			//create qualified tables
			QualifiedTable t1 = new BasicQualifiedTable("t1", null);
			QualifiedTable[] tables = {t1};
			//create a new leaf
			RdbExpQueryLeaf ql = new BasicRdbExpQueryLeaf(null, tables, "");
			//add fields
			RdbExpField f1 = ql.addRdbField("f1", null, "col1", t1);
			//create a filter to be added to the query leaf
			FieldOperand fieldOp1 = new BasicFieldOperand(f1);
			Collection col = new ArrayList();
			Operand colOp = null;
			if (isParameter)
				colOp = new BasicParameterOperand(PTypeSpaceMedor.COLLECTION, "param");
			else
				colOp = new BasicOperand(col, PTypeSpaceMedor.COLLECTION);
			Expression filter = new InCollection(fieldOp1, colOp, PTypeSpace.STRING);
			;
			//assign the filter to the query leaf
			ql.setQueryFilter(filter);
			//computes the SQL query
			String sqlrequest = null;
			logger.log(BasicLevel.DEBUG, "testEmptyCollection");
			if (isParameter) {
				ParameterOperand po = new BasicParameterOperand(
					PTypeSpaceMedor.COLLECTION, "param", col);
				sqlrequest = ql.getSqlRequest( new ParameterOperand[] {po});
			} else
				sqlrequest = ql.getSqlRequest(null);
			fail("Empty collection not detected");
		} catch (Exception e) {
			assertEquals("detected", e.getMessage(), "Empty collection for InCollection operator");
			logger.log(BasicLevel.DEBUG, "Empty collection detected");
		}
	}

	public void testCollOfStrings() {
		testCollOfStrings(false);
	}
	public void testParamCollOfStrings() {
		testCollOfStrings(true);
	}
	public void testCollOfStrings(boolean isParameter) {
		try {
			//create qualified tables
			QualifiedTable t1 = new BasicQualifiedTable("t1", null);
			QualifiedTable[] tables = {t1};
			//create a new leaf
			RdbExpQueryLeaf ql = new BasicRdbExpQueryLeaf(null, tables, "");
			//add fields
			RdbExpField f1 = ql.addRdbField("f1", null, "col1", t1);
			//create a filter to be added to the query leaf
			FieldOperand fieldOp1 = new BasicFieldOperand(f1);
			Collection col = new ArrayList();
			col.add("value1");
			col.add("val2");
			Operand colOp = null;
			if (isParameter)
				colOp = new BasicParameterOperand(PTypeSpaceMedor.COLLECTION, "param");
			else
				colOp = new BasicOperand(col, PTypeSpaceMedor.COLLECTION);
			Expression filter = new InCollection(fieldOp1, colOp, PTypeSpace.STRING);
			//assign the filter to the query leaf
			ql.setQueryFilter(filter);
			logger.log(BasicLevel.DEBUG, "Collection of Strings");
			//compute the SQL query
			String sqlrequest = null;
			if (isParameter) {
				ParameterOperand po = new BasicParameterOperand(
					PTypeSpaceMedor.COLLECTION, "param", col);
				sqlrequest = ql.getSqlRequest( new ParameterOperand[] {po});
			} else
				sqlrequest = ql.getSqlRequest(null);
			logger.log(BasicLevel.DEBUG, sqlrequest);
			assertEquals("Bad sql request", sqlrequest,
				"SELECT t1.col1 FROM t1 WHERE (t1.col1 IN ('value1', 'val2'))");
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	public void testCollOfFloats() {
		testCollOfFloats(false);
	}
	public void testParamCollOfFloats() {
		testCollOfFloats(true);
	}
	public void testCollOfFloats(boolean isParameter) {
		try {
			//create qualified tables
			QualifiedTable t1 = new BasicQualifiedTable("t1", null);
			QualifiedTable[] tables = {t1};
			//create a new leaf
			RdbExpQueryLeaf ql = new BasicRdbExpQueryLeaf(null, tables, "");
			//add fields
			RdbExpField f1 = ql.addRdbField("f1", null, "col1", t1);
			//create a filter to be added to the query leaf
			FieldOperand fieldOp1 = new BasicFieldOperand(f1);
			Collection col = new ArrayList();
			col.add(new Float(123.34));
			col.add(new Float(345.23));
			col.add(new BasicParameterOperand(PTypeSpaceMedor.FLOAT, "p1"));
			Operand colOp = null;
			if (isParameter)
				colOp = new BasicParameterOperand(PTypeSpaceMedor.COLLECTION, "param");
			else
				colOp = new BasicOperand(col, PTypeSpaceMedor.COLLECTION);
			Expression filter = new InCollection(fieldOp1, colOp, PTypeSpace.FLOAT);
			//assign the filter to the query leaf
			ql.setQueryFilter(filter);
			logger.log(BasicLevel.DEBUG, "Collection of floats");
			//compute the SQL query
			String sqlrequest = null;
			if (isParameter) {
				ParameterOperand po = new BasicParameterOperand(
					PTypeSpaceMedor.COLLECTION, "param", col);
				sqlrequest = ql.getSqlRequest( new ParameterOperand[] {po});
			} else
				sqlrequest = ql.getSqlRequest(null);
			logger.log(BasicLevel.DEBUG, sqlrequest);
			assertEquals("Bad sql request",
				"SELECT t1.col1 FROM t1 WHERE (t1.col1 IN (123.34, 345.23, ?))",
					sqlrequest);
			//testing evaluation
			/*
			IndexesGenerator ig = new IndexesGenerator();
			QueryTree qoptim = ig.transform(ql);
			new MedorEvaluator(qoptim, 0);
			*/

		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

		public void testIsNullPName() {
		try {
			//create qualified tables
			QualifiedTable t1 = new BasicQualifiedTable("t1", null);
			QualifiedTable[] tables = {t1};
			//create a new leaf
			RdbExpQueryLeaf ql = new BasicRdbExpQueryLeaf(null, tables, "");
			//add fields
			RdbExpField f1 = ql.addRdbField("f1", null, "col1", t1);
			//create a filter to be added to the query leaf
			FieldOperand fieldOp1 = new BasicFieldOperand(f1);
			BasicOperand valueL = new BasicOperand(1013);
			Expression left = new Lower(fieldOp1, valueL);
			Expression filter = new And(left, new IsNullPName("param1"));
			//assign the filter to the query leaf
			ql.setQueryFilter(filter);
			ql.setRdbAdapter(new PostgresAdapter());
			logger.log(BasicLevel.DEBUG, "testIsNullPName: Postgres");
			//compute the SQL query
			logger.log(BasicLevel.DEBUG, ql.getSqlRequest(null));
			assertEquals("Bad sql request",
				"SELECT t1.col1 FROM t1 WHERE ((t1.col1 < 1013) AND (? = TRUE))",
				ql.getSqlRequest(null));
			ql.setRdbAdapter(new OracleAdapter());
			ql.resetSqlRequest();
			logger.log(BasicLevel.DEBUG, "testIsNullPName: Oracle");
			//compute the SQL query
			logger.log(BasicLevel.DEBUG, ql.getSqlRequest(null));
			assertEquals("Bad sql request",
				"SELECT t1.col1 FROM t1 WHERE ((t1.col1 < 1013) AND (? = 1))",
				ql.getSqlRequest(null));
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}
}
